import { get, last } from 'lodash'

export const isHEToManuscript = (state, collectionId) => {
  const currentUserId = get(state, 'currentUser.user.id')
  const collections = get(state, 'collections', [])
  const collection = collections.find(c => c.id === collectionId) || {}
  return get(collection, 'handlingEditor.id') === currentUserId
}

const canMakeRecommendationStatuses = ['reviewCompleted', 'heAssigned']
export const canMakeRecommendation = (state, collection, fragment = {}) => {
  if (fragment.id !== last(collection.fragments)) return false
  const isHE = isHEToManuscript(state, collection.id)
  const status = get(collection, 'status')
  return isHE && canMakeRecommendationStatuses.includes(status)
}

export const currentUserIs = ({ currentUser: { user } }, role) => {
  const isAdmin = get(user, 'admin')
  const isEic = get(user, 'editorInChief')
  const isHe = get(user, 'handlingEditor')
  switch (role) {
    case 'isHE':
      return isHe
    case 'staff':
      return isAdmin || isEic || isHe
    case 'isEiC':
      return isEic
    case 'isAdmin':
      return isAdmin
    case 'adminEiC':
      return isAdmin || isEic
    default:
      return false
  }
}

export const canInviteReviewers = ({ currentUser: { user } }, project) => {
  const status = get(project, 'status')
  if (!status || status === 'rejected' || status === 'published') return false

  const handlingEditor = get(project, 'handlingEditor')
  const isAdmin = get(user, 'admin')
  const isEic = get(user, 'editorInChief')
  const isAccepted = get(handlingEditor, 'isAccepted')
  const heId = get(handlingEditor, 'id')
  return isAccepted && (user.id === heId || isAdmin || isEic)
}

export const getUserToken = ({ currentUser }) => get(currentUser, 'user.token')

export const getHERecommendation = (state, collectionId, fragmentId) => {
  const recommendations = get(
    state,
    `fragments.${fragmentId}.recommendations`,
    [],
  )
  const collection = get(state, 'collections').find(c => c.id === collectionId)
  const heId = get(collection, `handlingEditor.id`)

  return (
    recommendations.find(
      ({ userId, recommendationType }) =>
        recommendationType === 'editorRecommendation' && userId === heId,
    ) || {}
  )
}

const canMakeDecisionStatuses = ['submitted', 'pendingApproval']
export const canMakeDecision = (state, collection, fragment = {}) => {
  if (fragment.id !== last(collection.fragments)) return false
  const status = get(collection, 'status')

  const isEIC = currentUserIs(state, 'adminEiC')
  return isEIC && canMakeDecisionStatuses.includes(status)
}

const canEditManuscriptStatuses = ['draft', 'technicalChecks']
export const canEditManuscript = (state, collection, fragment = {}) => {
  if (fragment.id !== last(collection.fragments)) return false
  const status = get(collection, 'status')

  const isAdmin = currentUserIs(state, 'isAdmin')
  return isAdmin && canEditManuscriptStatuses.includes(status)
}

export const canSeeReviewersReports = (state, collectionId) => {
  const isHE = isHEToManuscript(state, collectionId)
  const isEiC = currentUserIs(state, 'adminEiC')
  return isHE || isEiC
}

export const canMakeRevision = (state, collection, fragment) => {
  const currentUserId = get(state, 'currentUser.user.id')
  return (
    get(fragment, 'revision') &&
    collection.status === 'revisionRequested' &&
    fragment.owners.map(o => o.id).includes(currentUserId)
  )
}

export const currentUserIsAuthor = (state, id) => {
  const permissions = getUserPermissions(state)

  return permissions
    .filter(f => f.role === 'author')
    .map(p => p.objectId)
    .includes(id)
}

export const getUserPermissions = ({ currentUser }) =>
  get(currentUser, 'user.teams', []).map(t => ({
    objectId: t.object.id,
    objectType: t.object.type,
    role: t.teamType.permissions,
  }))

export const userNotConfirmed = ({ currentUser }) =>
  get(currentUser, 'isAuthenticated') &&
  !currentUserIs({ currentUser }, 'staff') &&
  !get(currentUser, 'user.isConfirmed')
