const { get, isEmpty } = require('lodash')
const { services } = require('pubsweet-component-helper-service')

const { sendNotifications } = require('./notifications/notifications')

const TECHNICAL_STEPS = {
  EQS: 'eqs',
  EQA: 'eqa',
}

const setNewStatus = (step, agree) => {
  if (step === TECHNICAL_STEPS.EQS) {
    return agree ? 'submitted' : 'rejected'
  } else if (step === TECHNICAL_STEPS.EQA) {
    return agree ? 'accepted' : 'rejected'
  }
}

module.exports = ({ Collection, Fragment, User }) => async (req, res) => {
  const { collectionId } = req.params
  const { token, agree, step } = req.body

  try {
    const collection = await Collection.find(collectionId)
    const technicalCheckToken = get(collection, `technicalChecks.token`)

    if (isEmpty(technicalCheckToken)) {
      return res.status(400).json({
        error: `Manuscript already handled.`,
      })
    }

    if (technicalCheckToken !== token) {
      return res.status(400).json({
        error: `Invalid token.`,
      })
    }

    delete collection.technicalChecks.token
    collection.status = setNewStatus(step, agree)
    await collection.save()

    if (agree) {
      sendNotifications({
        User,
        Fragment,
        collection,
        baseUrl: services.getBaseUrl(req),
      })
    }

    return res.status(200).json(collection)
  } catch (e) {
    const notFoundError = await services.handleNotFoundError(e, 'Collection')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
